﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using MsBox.Avalonia.Enums;
using MsBox.Avalonia;
using NPOI.Util;
using SocialEducatorAvalonia.DTOs;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using SocialEducatorAvalonia.Models;

namespace SocialEducatorAvalonia.ViewModels
{
    public partial class AddGroupViewModel : ViewModelBase
    {
        [ObservableProperty]
        ObservableCollection<GroupItem> _groupsArrays = [];

        List<GroupItem> _groupStorage = [];

        [ObservableProperty]
        string? _newGroup = null;

        public AddGroupViewModel()
        {
            _groupStorage = DbContext.Groups.Select(x => new GroupItem { Id = x.GroupId, Title = x.Title }).ToList();

            GroupsArrays = new(_groupStorage.Copy());
        }

        [RelayCommand]
        void EditOrNot()
        {
            var list = GroupsArrays.ExceptBy(_groupStorage.Select(_ => _.Title), x => x.Title)
                .Select(x => new Group { GroupId = x.Id, Title = x.Title }).ToList();

            if (list.Count != 0) 
            {
                foreach (var item in list)
                {
                    var oldValue = DbContext.Groups.SingleOrDefault(x => x.GroupId == item.GroupId);

                    if (oldValue != null) 
                    {
                        oldValue.Title = item.Title;
                        DbContext.Groups.Update(oldValue);
                    }
                }

                DbContext.SaveChanges();

                _groupStorage = DbContext.Groups.Select(x => new GroupItem { Id = x.GroupId, Title = x.Title }).ToList();


                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Данные обновлены", ButtonEnum.Ok, Icon.Database).ShowAsync();
            }
            else
                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Данные не были подвержены изменениям", ButtonEnum.Ok, Icon.Error).ShowAsync();
        }


        [RelayCommand]
        void AddNewGroup()
        {
            if (!string.IsNullOrEmpty(NewGroup))
            {
                if (NewGroup.Length < 8)
                {
                    Group? newGroup = new() { Title = NewGroup };

                    DbContext.Groups.Add(newGroup);
                    DbContext.SaveChanges();

                    newGroup = DbContext.Groups.SingleOrDefault(x => x.Title == NewGroup);

                    if (newGroup != null)
                    {
                        GroupItem newGroupItem = new()
                        {
                            Id = newGroup.GroupId,
                            Title = newGroup.Title
                        };

                        _groupStorage.Add(newGroupItem);
                        GroupsArrays.Add(newGroupItem);
                    }

                    NewGroup = null;

                    MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Добавлено новое значение", ButtonEnum.Ok, Icon.Database).ShowAsync();
                }
                else
                    MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Слишком длинное название для группы", ButtonEnum.Ok, Icon.Error).ShowAsync();
            }
            else
                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Вы ничего не ввели, введите в поле значение", ButtonEnum.Ok, Icon.Error).ShowAsync();
        }

    }
}
