﻿using Avalonia.Controls;
using Avalonia.Platform.Storage;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using ExcelDataReader;
using MsBox.Avalonia.Enums;
using MsBox.Avalonia;
using SocialEducatorAvalonia.DTOs;
using SocialEducatorAvalonia.Utilities;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Collections.Immutable;
using Newtonsoft.Json;
using Formatting = Newtonsoft.Json.Formatting;
using Icon = MsBox.Avalonia.Enums.Icon;
using Microsoft.EntityFrameworkCore;
using Npgsql;
using NpgsqlTypes;
using Newtonsoft.Json.Linq;
using SocialEducatorAvalonia.Models;
using NPOI.Util;

namespace SocialEducatorAvalonia.ViewModels
{
    internal partial class AddStudentViewModel : ViewModelBase
    {
        #region arrays on page
        /// <summary>
        /// Коллекция отображаемая
        /// </summary>
        [ObservableProperty]
        ObservableCollection<Persons> _personsTable = [];
        /// <summary>
        /// В хранилище коллекция
        /// </summary>
        List<Persons> _personsInStorage = [];
        /// <summary>
        /// Статусы
        /// </summary>
        readonly ImmutableArray<string> _statusList;
        public ImmutableArray<string> StatusList { get => _statusList; }
        /// <summary>
        /// Группы
        /// </summary>
        readonly ImmutableArray<string> _groupList;
        public ImmutableArray<string> GroupList { get => _groupList; }

        /// <summary>
        /// Перваши которые приходят из GroupViewModel при переходе на следующий курс
        /// </summary>
        public List<Student> FirstGradeStudents { get; set; } = [];
        #endregion

        [RelayCommand]
        void GoBack()
        {
            if (FirstGradeStudents.Count == 0)
            {
                MainWindowViewModel.DataEditorTab = 1;
                MainVM.ChangeView(MainWindowViewModel.ListItems[0]);
            }
            else
                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Вы не сможете выйти так как вам нужно изменить данные", ButtonEnum.Ok, Icon.Info).ShowAsync();
        }

        [RelayCommand]
        private async Task OpenFileDialog(Window currentWindow)
        {
            if (currentWindow is not null)
            {
                IReadOnlyList<IStorageFile>? files = await ExcelUtilities.OpenFileDialogAsync(currentWindow);

                if (files is not null)
                    await FileWorkingAsync(files);
            }
        }

        public AddStudentViewModel()
        {
            _statusList = StatusDict.Keys.ToImmutableArray();
            _groupList = GroupDict.Keys.ToImmutableArray();
        }

        public void ParseStudents(ref List<Student> students)
        {
            FirstGradeStudents = students;

            var myCopy = students.Copy()
                .Select(x => new Persons
                {
                    Id = x.StudentId,
                    StudentGroupId = x.StudentGroupId,
                    StudentStatusId = x.StudentStatusId,
                    FIO = x.Fio,
                    Name = x.Name,
                    Surname = x.Surname,
                    Patronymic = x.Patronymic,
                    DataAdmission = x.AdmissionDate,
                    Group = GroupDict.FirstOrDefault(_ => _.Value == x.StudentGroupId).Key,
                    Status = StatusDict.FirstOrDefault(_ => _.Value == x.StudentStatusId).Key
                }).ToList();

            _personsInStorage = myCopy;
            PersonsTable = new(_personsInStorage);

            MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Вам нужно изменить все группы студентов в ручную", ButtonEnum.Ok, Icon.Info).ShowAsync();
        }

        void FillGroupAndStatus(Persons person)
        {
            // 4 - Б, В, П
            // 3 - С, Л, Ю

            if (person.Group.Contains('Б') || person.Group.Contains('В') || person.Group.Contains('П'))
            {
                if (person.Group[0] == '4')
                    person.Status = StatusList[2]; // Выпускник
                else
                    person.Status = StatusList[4]; // Обучающийся
            }
            else if (person.Group.Contains('С') || person.Group.Contains('Л') || person.Group.Contains('Ю'))
            {
                if (person.Group[0] == '3')
                    person.Status = StatusList[2];
                else
                    person.Status = StatusList[4];
            }
        }

        private async Task FileWorkingAsync(IReadOnlyList<IStorageFile> files)
        {
            _personsInStorage = [.. PersonsTable];

            if (files.Count >= 1)
            {
                if (ExcelUtilities.CheckFirstRow(ref files))
                {
                    for (int i = 0; i < files.Count; i++)
                    {
                        var data = new DataSet();

                        using (var stream = File.OpenRead(files[i].Path.LocalPath))
                        {
                            using (var reader = ExcelReaderFactory.CreateReader(stream))
                            {
                                // set the first row is name of column
                                var config = new ExcelDataSetConfiguration
                                {
                                    ConfigureDataTable = _ => new ExcelDataTableConfiguration
                                    {
                                        UseHeaderRow = true
                                    }
                                };

                                data = reader.AsDataSet(config);
                                reader.Close();
                            }
                            stream.Close();
                        }

                        ExcelUtilities.AddStudentsFioAndGroup(ref data, ref _personsInStorage);

                        foreach (Persons person in _personsInStorage)
                        {
                            FillGroupAndStatus(person);
                            person.DataAdmission = (short)((short)DateTime.Now.Year - (short)char.GetNumericValue(person.Group[0]));
                        }
                    }

                    PersonsTable = new(_personsInStorage.DistinctBy(x => new { x.FIO, x.Group }).AsEnumerable().Select(person =>
                    {
                        var spilledFio = person.FIO.Split(' ');
                        return new Persons
                        {
                            FIO = person.FIO,
                            Surname = spilledFio[0],
                            Name = spilledFio[1],
                            Patronymic = spilledFio[2],
                            Group = person.Group,
                            Status = person.Status,
                            DataAdmission = person.DataAdmission,
                            StudentGroupId = GroupDict[person.Group],
                            StudentStatusId = StatusDict[person.Status]
                        };
                    }).ToList());
                }
                else
                    await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Закройте файл", ButtonEnum.Ok, Icon.Error).ShowAsync();
            }
            else
                await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Файл не выбран", ButtonEnum.Ok, Icon.Error).ShowAsync();
        }

        [RelayCommand]
        void DeleteUser(Persons person) => ExcelUtilities.DeleteTheField(person, ref _personsTable);

        async Task SendDataToDb(ObservableCollection<Persons> persons)
        {
            var json = JsonConvert.SerializeObject(persons, Formatting.Indented);

            DbContext.Students.FromSql($"SELECT insert_missing_students({json})");

            string filePath = "..\\..\\..\\Properties\\launchSettings.json";

            // Чтение содержимого файла
            string jsonContent = File.ReadAllText(filePath);

            // Разбор JSON
            var launchSettings = JObject.Parse(jsonContent);
            var connectionString = launchSettings["ConnectionString"]?["SocialEducatorAvalonia"]?["ReleaseConnection"]?.Value<string>(); // was DefaultConnection - test

            using (var connection = new NpgsqlConnection(connectionString))
            {
                connection.Open();
                using (var command = new NpgsqlCommand("SELECT insert_missing_students(@json)", connection))
                {
                    command.Parameters.AddWithValue("@json", NpgsqlDbType.Jsonb, json);
                    command.ExecuteNonQuery();
                }
                connection.Close();
            }

            await DbContext.SaveChangesAsync();

            await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Данные были занесены", ButtonEnum.Ok, Icon.Database).ShowAsync();

            MainVM.SelectedListItem = MainWindowViewModel.ListItems[0];
        }


        [RelayCommand]
        async Task SaveDataIntoDb()
        {
            if (PersonsTable is not null && PersonsTable.Count >= 1)
            {
                if (FirstGradeStudents.Count >= 1)
                {
                    var newValues = PersonsTable.ExceptBy(FirstGradeStudents.Select(_ => _.Fio), x => x.FIO).ToList();
                    var firstGradeStudents = PersonsTable.Except(newValues).ToList();

                    bool allChanged = true;

                    foreach (var student in firstGradeStudents)
                    {
                        if (student.Group == "11П" || student.Group == "12П" || student.Group == "13П" || student.Group == "14П")
                        {
                            allChanged = false;
                            break;
                        }
                    }

                    if (allChanged)
                    {
                        // Все изменено и готово к обновлению

                        foreach (var student in firstGradeStudents)
                        {
                            Student? studentInList = FirstGradeStudents.FirstOrDefault(x => x.StudentId == student.Id);

                            if (studentInList is not null)
                                studentInList.StudentGroupId = GroupDict[student.Group];
                        }

                        DbContext.Students.UpdateRange(FirstGradeStudents);
                        DbContext.SaveChanges();

                        if (newValues is not null && newValues.Count >= 1 )
                            await SendDataToDb(new(newValues));

                        FirstGradeStudents.Clear();

                        await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Изменения были приняты", ButtonEnum.Ok, Icon.Success).ShowAsync();

                        MainVM.SelectedListItem = MainWindowViewModel.ListItems[0];
                    }
                    else
                        await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Убедитесь что у каждого студента изменилась группа", ButtonEnum.Ok, Icon.Error).ShowAsync();
                }
                else
                    await SendDataToDb(PersonsTable);
            }
            else
                await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Нету информации которую можно сохранить", ButtonEnum.Ok, Icon.Error).ShowAsync();
        }

        [RelayCommand]
        async Task AddRecordOfStudentAsync()
        {
            Persons student = new() { FIO = "Введите ФИО", Group = GroupList.Last(), Status = StatusList.Last(), DataAdmission = 2015 };

            if (PersonsTable.Count >= 0 && !PersonsTable.Contains(student))
                PersonsTable.Add(student);
            else
                await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Вы добавляете один и тот же элемент, " +
                           "пожалуйста измените информацию у {Введите ФИО, Нету, Отчислено, 2015}", ButtonEnum.Ok, Icon.Error).ShowAsync();
        }

        [RelayCommand]
        void ClearAllFields()
        {
            if (FirstGradeStudents.Count >= 1)
                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Вам нужно изменить данные всех первокурсников", ButtonEnum.Ok, Icon.Error).ShowAsync();
            else
            {
                PersonsTable.Clear();
                _personsInStorage.Clear();
            }            
        }

    }
}
