﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using MsBox.Avalonia;
using MsBox.Avalonia.Enums;
using NPOI.Util;
using SocialEducatorAvalonia.DTOs;
using SocialEducatorAvalonia.Models;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;

namespace SocialEducatorAvalonia.ViewModels
{
    public partial class AddTypeOfPaymentViewModel : ViewModelBase
    {
        [ObservableProperty]
        ObservableCollection<TypesOfPaymentsDTO> _typesOfPaymentsArray = [];

        List<TypesOfPaymentsDTO> typesOfPaymentsStorage = [];

        [ObservableProperty]
        [property: MinLength(10)]
        string? _newPayment = null;

        public AddTypeOfPaymentViewModel()
        {
            typesOfPaymentsStorage = DbContext.TypesOfPayments.ToList().Select(x => new TypesOfPaymentsDTO { Id = x.TypeOfPaymentId, TypeTittle = x.TypeTittle }).ToList();

            TypesOfPaymentsArray = new(typesOfPaymentsStorage.Copy());
        }

        [RelayCommand]
        void EditOrNot()
        {
            var list = TypesOfPaymentsArray.ExceptBy(typesOfPaymentsStorage.Select(_ => _.TypeTittle), x => x.TypeTittle)
                .Select(x => new TypesOfPayment { TypeOfPaymentId = x.Id, TypeTittle = x.TypeTittle }).ToList();

            if (list.Count != 0)
            {
                foreach (var item in list)
                {
                    var oldValue = DbContext.TypesOfPayments.SingleOrDefault(x => x.TypeOfPaymentId == item.TypeOfPaymentId);

                    if (oldValue is not null)
                    {
                        oldValue.TypeTittle = item.TypeTittle;
                        DbContext.TypesOfPayments.Update(oldValue);
                    }
                }

                DbContext.SaveChanges();

                typesOfPaymentsStorage = DbContext.TypesOfPayments.Select(x => new TypesOfPaymentsDTO { Id = x.TypeOfPaymentId, TypeTittle = x.TypeTittle }).ToList();

                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Значения обновлено", ButtonEnum.Ok, Icon.Database).ShowAsync();
            }
            else
                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Данные не были подвержены изменениям", ButtonEnum.Ok, Icon.Error).ShowAsync();
        }


        [RelayCommand]
        void AddNewPayment()
        {
            if (!string.IsNullOrEmpty(NewPayment))
            {
                TypesOfPayment? newPayment = new()
                {
                    TypeTittle = NewPayment
                };

                DbContext.TypesOfPayments.Add(newPayment);
                DbContext.SaveChanges();

                newPayment = DbContext.TypesOfPayments.SingleOrDefault(x => x.TypeTittle == NewPayment);

                if (newPayment != null)
                {
                    TypesOfPaymentsDTO newPaymentDTO = new()
                    {
                        Id = newPayment.TypeOfPaymentId,
                        TypeTittle = newPayment.TypeTittle,
                    };

                    typesOfPaymentsStorage.Add(newPaymentDTO);
                    TypesOfPaymentsArray.Add(newPaymentDTO);
                }

                NewPayment = null;

                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Добавлено новое значение", ButtonEnum.Ok, Icon.Database).ShowAsync();
            }
            else
                MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Вы ничего не ввели, введите в поле значение", ButtonEnum.Ok, Icon.Error).ShowAsync();
        }
    }
}
