﻿using CommunityToolkit.Mvvm.ComponentModel;
using SocialEducatorAvalonia.Utilities;
using System.Collections.ObjectModel;

namespace SocialEducatorAvalonia.ViewModels
{
    internal partial class DataEditorViewModel : ViewModelBase
    {
        /// <summary>
        /// Отображаемая коллекция вкладок
        /// </summary>
        readonly static ObservableCollection<TabListItem> _tabsCollection =
        [
            new TabListItem("Группы", new GroupViewModel()),
            new TabListItem("Студенты", new StudentsViewModel()),
            new TabListItem("Выплаты", new PaymentsViewModel())
        ];

        public static ObservableCollection<TabListItem> TabsCollection { get => _tabsCollection; }

        [ObservableProperty]
        TabListItem _selectedBtn = _tabsCollection[0];

        /// <summary>
        /// Вызов страницы со вкладками. С выборкой
        /// </summary>
        /// <param name="tab">Номер вкладки как в коллекции {0 - Группы, 1 - Студенты, 2 - Выплаты}</param>
        /// <exception cref="ArgumentOutOfRangeException">Выход за пределы коллекции</exception>
        public DataEditorViewModel(int tab)
        {
            switch (tab)
            {
                case 0:
                    SelectedBtn = TabsCollection[0];
                    break;
                case 1:
                    SelectedBtn = TabsCollection[1];
                    break;
                case 2:
                    SelectedBtn = TabsCollection[2];
                    break;
                default:
                    throw new ArgumentOutOfRangeException("Incorrect index of TabListItem");
            }
        }
    }
}
