﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.EntityFrameworkCore;
using MsBox.Avalonia.Enums;
using MsBox.Avalonia;
using SocialEducatorAvalonia.DTOs;
using SocialEducatorAvalonia.Models;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace SocialEducatorAvalonia.ViewModels
{
    internal partial class GroupViewModel : ViewModelBase
    {
        #region lists with data On page
        [ObservableProperty]
        List<GroupItem> _groupArray = new();

        List<GroupItem> _groupStorage = new();

        [ObservableProperty]
        string[] _groupsForBox = [];
        #endregion

        [ObservableProperty]
        string? _groupSelected = null;
        
        partial void OnGroupSelectedChanged(string? value)
        {
            GroupArray = _groupStorage;

            if (!string.IsNullOrWhiteSpace(value))
                GroupArray = GroupArray.Where(x => x.Title.ToLower().Contains(value.ToLower())).ToList();
        }

        public GroupViewModel()
        {
            var groupWithData = DbContext.ViewCountStudentsInGroups.AsEnumerable().Select(row => new GroupItem
            {
                Title = GroupDict.FirstOrDefault(x => x.Value == row.StudentGroupId).Key,
                CountsOfPerson = (int)row.StudentsCount!
            }).ToList();

            if (groupWithData is not null)
            {
                GroupArray = groupWithData;
                _groupStorage = groupWithData;
                GroupsForBox = groupWithData.AsEnumerable().Select(x => x.Title).ToArray();
            }
        }

        [RelayCommand]
        void GoToStudents(GroupItem currentGroup)
        {
            var mvm = MainWindowViewModel.ListItems[0];
            MainWindowViewModel.DataEditorTab = 1;

            MainVM.ChangeView(mvm);

            if (mvm.Page is DataEditorViewModel vm)
            {
                if (vm.SelectedBtn.TabContent is StudentsViewModel vm2)
                    vm2.SelectedGroup = currentGroup.Title;
            }
        }

        [RelayCommand]
        void GoToPayments(GroupItem currentGroup)
        {
            var mvm = MainWindowViewModel.ListItems[0];
            MainWindowViewModel.DataEditorTab = 2;

            MainVM.ChangeView(mvm);

            if (mvm.Page is DataEditorViewModel vm)
            {
                if (vm.SelectedBtn.TabContent is PaymentsViewModel vm2)
                    vm2.SelectedGroup = currentGroup.Title;
            }
        }


        void ManipulationWithGroup(Student student, ref string valueOfGroup, bool brancher)
        {
            if (brancher)
            {
                // выпускник
                student.StudentGroupId = 45; // Нет группы
                student.StudentStatusId = 6;
            }
            else
            {
                string newCourse = (char.GetNumericValue(valueOfGroup[0]) + 1).ToString();
                valueOfGroup = valueOfGroup.Replace(valueOfGroup[0], newCourse[0]);

                student.StudentGroupId = GroupDict[valueOfGroup];
            }
        }

        [RelayCommand]
        async Task TransferOfStudents()
        {
            var box = MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Данные будут обновлены вы готовы к этому шагу?", ButtonEnum.YesNo, Icon.Success);
            ButtonResult response = await box.ShowAsync();

            if (response == ButtonResult.Yes)
            {
                var list = DbContext.Students.FromSql($"select * from students_active_and_graduation").ToList();

                // Берем первый курс программистов чтобы фасовать их вручную
                var listProgrammersFirstGrade = list.Where(x => x.StudentGroupId >= 27 && x.StudentGroupId <= 30).ToList();

                list = list.Except(listProgrammersFirstGrade).ToList();

                foreach (Student student in list)
                {
                    string? valueOfGroup = GroupDict.FirstOrDefault(x => x.Value == student.StudentGroupId).Key;

                    if (valueOfGroup != null)
                    {
                        string onlyLetters = Regex.Replace(valueOfGroup, "\\d", "");

                        if (onlyLetters.Length == 1)
                        {
                            char letter = onlyLetters.First();

                            if (letter == 'Б' || letter == 'В' || letter == 'П')
                            {
                                // до 4 курса
                                if (valueOfGroup[0] >= '4')
                                    ManipulationWithGroup(student, ref valueOfGroup, true);
                                else
                                    ManipulationWithGroup(student, ref valueOfGroup, false);

                            }
                            else if (letter == 'С' || letter == 'Л' || letter == 'Ю')
                            {
                                // до 3 курса
                                if (valueOfGroup[0] >= '3')
                                    ManipulationWithGroup(student, ref valueOfGroup, true);
                                else
                                    ManipulationWithGroup(student, ref valueOfGroup, false);
                            }
                        }
                    }
                }

                DbContext.Students.UpdateRange(list);
                DbContext.SaveChanges();

                await MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Данные были обновлены", ButtonEnum.Ok, Icon.Success).ShowAsync();

                if (listProgrammersFirstGrade.Count > 0)
                {
                    // Отдаем весь список и как то обрабатываем его на другой форме

                    var mvm = MainWindowViewModel.ListItems[1];
                    MainWindowViewModel.AddDataTab = 1;

                    MainVM.ChangeView(mvm);

                    if (mvm.Page is AddDataToDbViewModel vm)
                    {
                        if (vm.SelectedBtn.TabContent is AddStudentViewModel vm2)
                            vm2.ParseStudents(ref listProgrammersFirstGrade);
                    }

                    //PageSwitcher.View = new AddStudentViewModel(ref listProgrammersFirstGrade);
                }
            }         
        }

    }
}