﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using MsBox.Avalonia.Enums;
using MsBox.Avalonia;
using SocialEducatorAvalonia.Models;
using SocialEducatorAvalonia.Utilities;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.Collections.ObjectModel;
using System.Linq;
using Microsoft.EntityFrameworkCore;
using System.Diagnostics;
using Org.BouncyCastle.Security.Certificates;

namespace SocialEducatorAvalonia.ViewModels
{
    public partial class PaymentsViewModel : ViewModelBase
    {
        #region arrays
        readonly ImmutableArray<string> _paymentsType;
        public ImmutableArray<string> PaymentsType { get => _paymentsType; }

        readonly ImmutableArray<string> _groups;
        public ImmutableArray<string> Groups { get => _groups; }

        /// <summary>
        /// Перечисление статусов (Без Обучающийся)
        /// </summary>
        readonly ImmutableArray<string> _statusList;
        public ImmutableArray<string> StatusList { get => _statusList; }
        #endregion

        #region selectors
        [ObservableProperty]
        static string? _selectedFIO = null;

        [ObservableProperty]
        static string? _selectedGroup = null;

        [ObservableProperty]
        static string _selectedPayment = "Не выбрано";

        [ObservableProperty]
        string? _selectedStatus = null;

        [ObservableProperty]
        DateTime? _selectedStartDate = null;

        [ObservableProperty]
        DateTime? _selectedEndDate = null;

        [ObservableProperty]
        bool _selectedComboBox = false;
        #endregion

        readonly Dictionary<string, int> PaymentDict = [];

        /// <summary>
        /// перечисление студентов в кэше
        /// </summary>
        List<Student> _studentsStorage = [];

        /// <summary>
        /// Отображаемая на странице коллекция
        /// </summary>
        [ObservableProperty]
        ObservableCollection<Student> _studentsList = [];

        /// <summary>
        /// Список идущий в выплаты
        /// </summary>
        static List<Student> _personsToPayment = [];

        /// <summary>
        /// Проверяет все поля на наличие значений
        /// </summary>
        /// <returns>True - могу менять, False - не могу менять</returns>
        bool CheckAllFields()
        {
            string functionName = new StackTrace().GetFrame(1)?.GetMethod()?.Name ?? "Unknown";

            switch (functionName)
            {
                case "OnSelectedFIOChanged":
                    if (!string.IsNullOrEmpty(SelectedGroup) || SelectedPayment is not "Не выбрано" ||
                        SelectedStartDate is not null || SelectedEndDate is not null || !string.IsNullOrEmpty(SelectedStatus))
                        return false; // Если значение заполнено
                    break;

                case "OnSelectedGroupChanged":
                    if (SelectedPayment is not "Не выбрано" || !string.IsNullOrEmpty(SelectedFIO) ||
                        SelectedStartDate.HasValue || SelectedEndDate.HasValue || !string.IsNullOrEmpty(SelectedStatus))
                        return false;
                    break;

                case "OnSelectedPaymentChanged":
                    if (!string.IsNullOrEmpty(SelectedGroup) || !string.IsNullOrEmpty(SelectedFIO) ||
                        SelectedStartDate.HasValue || SelectedEndDate.HasValue || !string.IsNullOrEmpty(SelectedStatus))
                        return false;
                    break;

                case "FindByDates":
                    if (SelectedPayment is not "Не выбрано" || !string.IsNullOrEmpty(SelectedGroup)
                        || !string.IsNullOrEmpty(SelectedFIO) || !string.IsNullOrEmpty(SelectedStatus))
                        return false;
                    break;

                case "OnSelectedStatusChanged":
                    if (SelectedPayment is not "Не выбрано" || !string.IsNullOrEmpty(SelectedGroup) || !string.IsNullOrEmpty(SelectedFIO) ||
                        SelectedStartDate.HasValue || SelectedEndDate.HasValue)
                        return false;
                    break;
            }

            return true;
        }

        /// <summary>
        /// Выбирает метод который следует применять исходя из отображаемой информации
        /// </summary>
        void TakeCorrectData()
        {
            string functionName = new StackTrace().GetFrame(1)?.GetMethod()?.Name ?? "Unknown";

            switch (functionName)
            {
                case "OnSelectedFIOChanged":
                    if (!string.IsNullOrEmpty(SelectedGroup))
                        OnSelectedGroupChanged(SelectedGroup);

                    if (SelectedPayment is not "Не выбрано")
                        OnSelectedPaymentChanged(SelectedPayment);

                    if (!string.IsNullOrEmpty(SelectedStatus))
                        OnSelectedStatusChanged(SelectedStatus);

                    if (SelectedStartDate.HasValue || SelectedEndDate.HasValue)
                        FindByDates();
                    break;

                case "OnSelectedGroupChanged":
                    if (!string.IsNullOrEmpty(SelectedFIO))
                        OnSelectedFIOChanged(SelectedFIO);

                    if (SelectedPayment is not "Не выбрано")
                        OnSelectedPaymentChanged(SelectedPayment);

                    if (!string.IsNullOrEmpty(SelectedStatus))
                        OnSelectedStatusChanged(SelectedStatus);

                    if (SelectedStartDate.HasValue || SelectedEndDate.HasValue)
                        FindByDates();
                    break;

                case "OnSelectedPaymentChanged":

                    if (!string.IsNullOrEmpty(SelectedGroup))
                        OnSelectedGroupChanged(SelectedGroup);

                    if (!string.IsNullOrEmpty(SelectedFIO))
                        OnSelectedFIOChanged(SelectedFIO);

                    if (!string.IsNullOrEmpty(SelectedStatus))
                        OnSelectedStatusChanged(SelectedStatus);

                    if (SelectedStartDate.HasValue || SelectedEndDate.HasValue)
                        FindByDates();
                    break;

                case "FindByDates":

                    if (!string.IsNullOrEmpty(SelectedGroup))
                        OnSelectedGroupChanged(SelectedGroup);

                    if (!string.IsNullOrEmpty(SelectedFIO))
                        OnSelectedFIOChanged(SelectedFIO);

                    if (SelectedPayment is not "Не выбрано")
                        OnSelectedPaymentChanged(SelectedPayment);

                    if (!string.IsNullOrEmpty(SelectedStatus))
                        OnSelectedStatusChanged(SelectedStatus);

                    break;

                case "OnSelectedStatusChanged":

                    if (!string.IsNullOrEmpty(SelectedGroup))
                        OnSelectedGroupChanged(SelectedGroup);

                    if (!string.IsNullOrEmpty(SelectedFIO))
                        OnSelectedFIOChanged(SelectedFIO);

                    if (SelectedPayment is not "Не выбрано")
                        OnSelectedPaymentChanged(SelectedPayment);

                    if (SelectedStartDate.HasValue || SelectedEndDate.HasValue)
                        FindByDates();

                    break;
            }

            if (string.IsNullOrEmpty(SelectedGroup) && string.IsNullOrEmpty(SelectedFIO)
                && SelectedPayment is "Не выбрано" && !SelectedStartDate.HasValue && !SelectedEndDate.HasValue
                && string.IsNullOrEmpty(SelectedStatus))
            {
                var list = new ObservableCollection<Student>(DbContext.StudentsActiveAndGraduations.Include(_ => _.StudentStatus).Include(_ => _.StudentGroup)
                .Take(200).AsEnumerable().Select(x =>
                {
                    string[] spilledFio = x.Fio.Split(' ');
                    var student = new Student
                    {
                        StudentId = x.StudentId,
                        Fio = x.Fio,
                        Surname = spilledFio.Length > 0 ? spilledFio[0] : string.Empty,
                        Name = spilledFio.Length > 1 ? spilledFio[1] : string.Empty,
                        Patronymic = spilledFio.Length > 2 ? spilledFio[2] : string.Empty,
                        AdmissionDate = x.AdmissionDate,
                        StudentGroupId = x.StudentGroupId,
                        StudentStatusId = x.StudentStatusId,
                        StudentGroup = x.StudentGroup,
                        StudentStatus = x.StudentStatus
                    };

                    return student;
                }).AsQueryable().Include(x => x.StudentsAndPayments));

                UpdateCollections(ref list);
            }

        }

        partial void OnSelectedFIOChanged(string? value)
        {
            SelectedFIO = value;

            ObservableCollection<Student> students = new();

            if (!string.IsNullOrEmpty(value))
            {
                if (CheckAllFields())
                    students = new(DbContext.Students.FromSql($"select * from search_students_by_fio({value})").Include(x => x.StudentsAndPayments).ToList());
                else
                    students = new(StudentsList.Where(x => x.Fio!.ToLower().Contains(value.ToLower())).ToList());
            }
            else // значение пустое
                TakeCorrectData();

            if (students.Count > 0)
                UpdateCollections(ref students);
        }

        partial void OnSelectedGroupChanged(string? value)
        {
            SelectedGroup = value;

            var collection = new ObservableCollection<Student>();

            if (value is not null)
            {
                int groupId;
                GroupDict.TryGetValue(value, out groupId);

                if (CheckAllFields()) // можем изменять
                {
                    int studentCount = DbContext.Database.SqlQuery<int>($"select get_count_students_by_group({groupId})").AsEnumerable().FirstOrDefault();
                    int studentsInStorage = _studentsStorage.Where(x => x.StudentGroupId == groupId).Count();

                    if (studentCount == studentsInStorage)
                        collection = new ObservableCollection<Student>(_studentsStorage.Where(x => x.StudentGroupId == groupId).ToList());
                    else if (studentCount == 0)
                    {
                        MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "В данной группе нету студентов", ButtonEnum.Ok, Icon.Database).ShowAsync();
                        collection = null;
                    }
                    else
                        collection = new(DbContext.Students.FromSql($"SELECT * FROM get_students_by_group_id({groupId})").Include(x => x.StudentsAndPayments).ToList());
                }
                else
                {
                    int studentsCount = StudentsList.Where(x => x.StudentGroupId == groupId).Count();

                    if (studentsCount > 0)
                        collection = new(StudentsList.Where(_ => _.StudentGroupId == groupId));
                    else
                    {
                        MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "С данным условием не существует студентов при данной группой", ButtonEnum.Ok, Icon.Database).ShowAsync();
                        collection = null;
                    }
                }
            }
            else // значение пустое
                TakeCorrectData();

            if (collection is not null && collection.Count > 0)
                UpdateCollections(ref collection);
        }

        partial void OnSelectedPaymentChanged(string value)
        {
            SelectedPayment = value;

            var collection = new ObservableCollection<Student>();

            if (value is not "Не выбрано")
            {
                if (CheckAllFields()) // могу менять                    
                    collection = new(DbContext.StudentsAndPayments.Where(x => x.TypePayment.TypeTittle == value).Select(x => x.Student).Distinct());
                else // не могу менять
                    collection = new(StudentsList.Where(x => x.StudentsAndPayments.Where(y => y.TypePayment.TypeTittle == value).Count() >= 1).ToList());

                if (collection.Count == 0)
                    MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Нету записей с данной выплатой", ButtonEnum.Ok, Icon.Database).ShowAsync();
            }
            else // Типа пустое, то есть поиск сбрасывается
                TakeCorrectData();

            if (collection.Count > 0)
                UpdateCollections(ref collection);
        }

        partial void OnSelectedStatusChanged(string? value)
        {
            SelectedStatus = value;

            var collection = new ObservableCollection<Student>();

            if (value is not null)
            {
                int statusId;
                StatusDict.TryGetValue(value, out statusId);

                if (CheckAllFields())
                    collection = new(DbContext.Students.FromSql($"SELECT * FROM get_students_by_status({statusId})"));
                else
                {
                    int countOfStudents = StudentsList.Where(x => x.StudentStatusId == statusId).Count();

                    if (countOfStudents > 0)
                        collection = new(StudentsList.Where(x => x.StudentStatusId == statusId));
                    else
                        MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "С данным условием не существует студентов с данным статусом", ButtonEnum.Ok, Icon.Database).ShowAsync();
                }
            }
            else
                TakeCorrectData();

            if (collection is not null && collection.Count > 0)
                UpdateCollections(ref collection);           
        }

        [RelayCommand]
        void FindByDates()
        {
            var collection = new ObservableCollection<Student>();

            if (SelectedStartDate.HasValue && SelectedEndDate.HasValue) // два не пустых
            {
                if (CheckAllFields()) // можем брать из БД
                {
                    DateOnly StartedDate = DateOnly.FromDateTime((DateTime)SelectedStartDate);
                    DateOnly EndDate = DateOnly.FromDateTime((DateTime)SelectedEndDate);

                    if (StartedDate.CompareTo(EndDate) < 0)
                        collection = new([.. DbContext.Students.FromSql($"SELECT * FROM find_students_by_payment_date_range({StartedDate}, {EndDate})").Include(x => x.StudentsAndPayments)]);
                    else
                        MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Левое значение должно быть меньше правого в дате", ButtonEnum.Ok, Icon.Warning).ShowAsync();
                }
                else
                    TakeCorrectData();
            }
            else // одно из значений пустое или все пустое
            {
                if (SelectedStartDate.HasValue == SelectedEndDate.HasValue)
                {
                    TakeCorrectData();
                    MessageBoxManager.GetMessageBoxStandard("Сообщение для вас", "Нет не одной даты", ButtonEnum.Ok, Icon.Warning).ShowAsync();
                }
                else
                {
                    if (SelectedStartDate.HasValue)
                    {
                        DateOnly StartedDate = DateOnly.FromDateTime((DateTime)SelectedStartDate);

                        if (CheckAllFields())
                            collection = new(DbContext.Students.FromSql($"select * from find_students_by_payment_start_date({StartedDate})").Include(x => x.StudentsAndPayments).ToList());
                        else
                            collection = new(StudentsList.Where(x => x.StudentsAndPayments.All(y => y.DataOfPayment.CompareTo(StartedDate) >= 0)).ToList());
                    }
                    else if (SelectedEndDate.HasValue)
                    {
                        DateOnly EndDate = SelectedEndDate.HasValue ? DateOnly.FromDateTime(SelectedEndDate.Value) : default;

                        if (CheckAllFields())
                            collection = new([.. DbContext.Students.FromSql($"select * from find_students_by_payment_end_date({EndDate})").Include(x => x.StudentsAndPayments)]);
                        else
                            collection = new(StudentsList.Where(x => x.StudentsAndPayments.All(y => y.DataOfPayment.CompareTo(EndDate) <= 0)).ToList());

                    }
                }
            }

            if (collection.Count > 0)
                UpdateCollections(ref collection);
        }

        #region select all
        /// <summary>
        /// Когда меняю страницу
        /// </summary>
        /// <param name="value"></param>
        partial void OnStudentsListChanged(ObservableCollection<Student> value)
        {
            StudentsList = value;

            var itemNotInList = StudentsList.FirstOrDefault(x => x.IsChecked == false);

            if (itemNotInList is not null)
                SelectedComboBox = false;
            else
                SelectedComboBox = true;
        }

        /// <summary>
        /// Изменять статус добавления в лист
        /// </summary>
        /// <param name="value">значение поля CheckBox</param>
        void ChangeCheckedStatusOnList(bool value)
        {
            foreach (var item in StudentsList)
            {
                item.IsChecked = value;
            }
        }

        [RelayCommand]
        void CheckBoxClicked()
        {
            if (!SelectedComboBox)
            {
                // Убрать из списка те которые есть на этой странице
                _personsToPayment = _personsToPayment.Except(StudentsList).ToList();
                SelectedComboBox = false;
            }
            else
            {
                // Добавить в список
                _personsToPayment.AddRange(StudentsList.Except(_personsToPayment).ToList());
                SelectedComboBox = true;
            }
            ChangeCheckedStatusOnList(SelectedComboBox);
        }
        #endregion

        #region pagination
        /// <summary>
        /// Текущая позиция по выборке из 25
        /// </summary>
        [ObservableProperty]
        int _currentPage = 1;

        /// <summary>
        /// Всего элементов, с условием что мы делим на 25
        /// </summary>
        [ObservableProperty]
        int _pageSize = 0;

        /// <summary>
        /// Обновление отображаемой коллекции
        /// </summary>
        private void UpdateViewCollection()
        {
            int skip = (CurrentPage - 1) * 25;

            var students = _studentsStorage.Skip(skip).Take(25).ToList();

            foreach (var student in students)
            {
                var currentStudent = _personsToPayment.SingleOrDefault(x => x.Equals(student));

                if (currentStudent is not null)
                    student.IsChecked = true;
            }

            StudentsList = new(students);
        }

        /// <summary>
        /// Команда срабатывающая на кнопку -> 
        /// </summary>
        [RelayCommand]
        void GoNextPage()
        {
            if (CurrentPage + 1 <= PageSize)
            {
                CurrentPage++;
                UpdateViewCollection();
            }
        }

        /// <summary>
        /// Команда срабатывающая на кнопку <-
        /// </summary>
        [RelayCommand]
        void GoPreviousPage()
        {
            if (CurrentPage - 1 >= 1)
            {
                CurrentPage--;
                UpdateViewCollection();
            }
        }

        /// <summary>
        /// Обновить элементы пагинации
        /// </summary>
        private void UpdateCountOfPages()
        {
            PageSize = (int)Math.Ceiling((decimal)_studentsStorage.Count / (decimal)25);
            CurrentPage = 1;
        }

        /// <summary>
        /// Обновление хранимой в кеш коллекции + страничной
        /// </summary>
        /// <param name="collection">Коллекция из запроса</param>
        private void UpdateCollections(ref ObservableCollection<Student> collection)
        {
            if (collection.Count > 25)
            {
                _studentsStorage = [.. collection];
                UpdateCountOfPages();
                UpdateViewCollection();
            }
            else
            {
                StudentsList = collection;
                PageSize = 1;
            }
        }
        #endregion

        public PaymentsViewModel()
        {
            _groups = GroupDict.Keys.ToImmutableArray();
            PaymentDict = DbContext.TypesOfPayments.OrderBy(x => x.TypeTittle).ToDictionary(x => x.TypeTittle.ToString(), x => (int)x.TypeOfPaymentId);

            _paymentsType = PaymentDict.Keys.ToImmutableArray().Add("Не выбрано");

            _studentsStorage = DbContext.StudentsActiveAndGraduations.Include(_ => _.StudentStatus).Include(_ => _.StudentGroup)
                .Take(200).AsEnumerable().Select(x =>
                {
                    string[] spilledFio = x.Fio.Split(' ');
                    var student = new Student
                    {
                        StudentId = x.StudentId,
                        Fio = x.Fio,
                        Surname = spilledFio.Length > 0 ? spilledFio[0] : string.Empty,
                        Name = spilledFio.Length > 1 ? spilledFio[1] : string.Empty,
                        Patronymic = spilledFio.Length > 2 ? spilledFio[2] : string.Empty,
                        AdmissionDate = x.AdmissionDate,
                        StudentGroupId = x.StudentGroupId,
                        StudentStatusId = x.StudentStatusId,
                        StudentGroup = x.StudentGroup,
                        StudentStatus = x.StudentStatus
                    };

                    return student;
                }).AsQueryable().Include(x => x.StudentsAndPayments).ToList();

            StudentsList = new(_studentsStorage.Take(25));

            _statusList = StatusDict.Keys.Where((v, i) => v != "Обучается").ToImmutableArray();

            UpdateCountOfPages();
        }

        [RelayCommand]
        void AddToList(Student student)
        {
            var addedOrNot = _personsToPayment.SingleOrDefault(x => x.Equals(student));

            if (addedOrNot is not null)
                _personsToPayment.Remove(addedOrNot);
            else
                _personsToPayment.Add(student);

            var changeComboBox = StudentsList.FirstOrDefault(x => x.IsChecked == false);

            if (changeComboBox is not null)
                SelectedComboBox = false;
            else
                SelectedComboBox = true;
        }

        #region navigation
        [RelayCommand]
        void GoToPersonalCabinet(Student student) => PageSwitcher.View = new PersonalStudentPaymentViewModel(student);

        [RelayCommand]
        void GoToAddPayments()
        {
            _personsToPayment.ForEach(_ => _.IsChecked = false);

            AddPaymentsViewModel.TakeList(_personsToPayment);

            MainWindowViewModel.AddDataTab = 1;
            MainVM.ChangeView(MainWindowViewModel.ListItems[1]);

            _personsToPayment.Clear();
            SelectedComboBox = false;
        }
        #endregion
    }
}
